@@ -1,9 +1,9 @@
-/* $Id: color.c 4540 2011-03-05 05:01:13Z astyanax $ */
+/* $Id: color.c 4894 2014-05-16 10:34:05Z bens $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
- *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009   *
- *   Free Software Foundation, Inc.                                       *
+ *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,  *
+ *   2010, 2011, 2013, 2014 Free Software Foundation, Inc.                *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 3, or (at your option)  *
@@ -31,17 +31,53 @@
 #include <magic.h>
 #endif
 
-#ifdef ENABLE_COLOR
+#ifndef DISABLE_COLOR
 
 /* For each syntax list entry, go through the list of colors and assign
  * the color pairs. */
 void set_colorpairs(void)
 {
     const syntaxtype *this_syntax = syntaxes;
+    bool defok = FALSE;
+    short fg, bg;
+    size_t i;
+
+    start_color();
+
+#ifdef HAVE_USE_DEFAULT_COLORS
+    /* Use the default colors, if available. */
+    defok = (use_default_colors() != ERR);
+#endif
+
+    for (i = 0; i < NUMBER_OF_ELEMENTS; i++) {
+	bool bright = FALSE;
+
+	if (parse_color_names(specified_color_combo[i], &fg, &bg, &bright)) {
+	    if (fg == -1 && !defok)
+		fg = COLOR_WHITE;
+	    if (bg == -1 && !defok)
+		bg = COLOR_BLACK;
+	    init_pair(i + 1, fg, bg);
+	    interface_color_pair[i].bright = bright;
+	    interface_color_pair[i].pairnum = COLOR_PAIR(i + 1);
+	}
+	else {
+	    interface_color_pair[i].bright = FALSE;
+	    if (i != FUNCTION_TAG)
+		interface_color_pair[i].pairnum = hilite_attribute;
+	    else
+		interface_color_pair[i].pairnum = A_NORMAL;
+	}
+
+	if (specified_color_combo[i] != NULL) {
+	    free(specified_color_combo[i]);
+	    specified_color_combo[i] = NULL;
+	}
+    }
 
     for (; this_syntax != NULL; this_syntax = this_syntax->next) {
 	colortype *this_color = this_syntax->color;
-	int color_pair = 1;
+	int clr_pair = NUMBER_OF_ELEMENTS + 1;
 
 	for (; this_color != NULL; this_color = this_color->next) {
 	    const colortype *beforenow = this_syntax->color;
@@ -56,8 +92,8 @@ void set_colorpairs(void)
 	    if (beforenow != this_color)
 		this_color->pairnum = beforenow->pairnum;
 	    else {
-		this_color->pairnum = color_pair;
-		color_pair++;
+		this_color->pairnum = clr_pair;
+		clr_pair++;
 	    }
 	}
     }
@@ -70,15 +106,9 @@ void color_init(void)
 
     if (has_colors()) {
 	const colortype *tmpcolor;
-#ifdef HAVE_USE_DEFAULT_COLORS
-	bool defok;
-#endif
-
-	start_color();
-
 #ifdef HAVE_USE_DEFAULT_COLORS
 	/* Use the default colors, if available. */
-	defok = (use_default_colors() != ERR);
+	bool defok = (use_default_colors() != ERR);
 #endif
 
 	for (tmpcolor = openfile->colorstrings; tmpcolor != NULL;
@@ -107,7 +137,7 @@ void color_init(void)
     }
 }
 
-/* Cleanup a regex we previously compiled */
+/* Clean up a regex we previously compiled. */
 void nfreeregex(regex_t **r)
 {
     assert(r != NULL);
@@ -123,17 +153,14 @@ void color_update(void)
     syntaxtype *tmpsyntax;
     syntaxtype *defsyntax = NULL;
     colortype *tmpcolor, *defcolor = NULL;
-    exttype *e;
+    regexlisttype *e;
 
-/* libmagic structures */
-/* magicstring will be NULL if we fail to get magic result */
+/* Var magicstring will stay NULL if we fail to get a magic result. */
 #ifdef HAVE_LIBMAGIC
     const char *magicstring = NULL;
-    const char *magicerr = NULL;
-    magic_t m;
+    magic_t cookie = NULL;
     struct stat fileinfo;
-#endif /* HAVE_LIBMAGIC */
-
+#endif
 
     assert(openfile != NULL);
 
@@ -159,32 +186,9 @@ void color_update(void)
 	}
     }
 
-#ifdef HAVE_LIBMAGIC
-
-    if (strcmp(openfile->filename,"") && stat(openfile->filename, &fileinfo) == 0) {
-	m = magic_open(MAGIC_SYMLINK |
-#ifdef DEBUG
-                       MAGIC_DEBUG | MAGIC_CHECK |
-#endif /* DEBUG */
-                       MAGIC_ERROR);
-	if (m == NULL || magic_load(m, NULL) < 0)
-	    fprintf(stderr, "something went wrong: %s [%s]\n", strerror(errno), openfile->filename);
-	else {
-	    magicstring = magic_file(m,openfile->filename);
-	    if (magicstring == NULL) {
-		magicerr = magic_error(m);
-		fprintf(stderr, "something went wrong: %s [%s]\n", magicerr, openfile->filename);
-            }
-#ifdef DEBUG
-	    fprintf(stderr, "magic string returned: %s\n", magicstring);
-#endif /* DEBUG */
-	}
-    }
-#endif /* HAVE_LIBMAGIC */
-
     /* If we didn't specify a syntax override string, or if we did and
      * there was no syntax by that name, get the syntax based on the
-     * file extension, and then look in the header. */
+     * file extension, then try the headerline, and then try magic. */
     if (openfile->colorstrings == NULL) {
 	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 		tmpsyntax = tmpsyntax->next) {
@@ -209,8 +213,7 @@ void color_update(void)
 		    regcomp(e->ext, fixbounds(e->ext_regex), REG_EXTENDED);
 		}
 
-		/* Set colorstrings if we matched the extension
-		 * regex. */
+		/* Set colorstrings if we match the extension regex. */
 		if (regexec(e->ext, openfile->filename, 0, NULL, 0) == 0) {
 		    openfile->syntax = tmpsyntax;
 		    openfile->colorstrings = tmpsyntax->color;
@@ -224,26 +227,80 @@ void color_update(void)
 	    }
 	}
 
-	    /* Check magic if we don't yet have an answer */
-#ifdef HAVE_LIBMAGIC
+	/* Check the headerline if the extension didn't match anything. */
 	if (openfile->colorstrings == NULL) {
-
 #ifdef DEBUG
-	    fprintf(stderr, "No match using extension, trying libmagic...\n");
-#endif /* DEBUG */
-
+	    fprintf(stderr, "No result from file extension, trying headerline...\n");
+#endif
 	    for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 		tmpsyntax = tmpsyntax->next) {
-		for (e = tmpsyntax->magics; e != NULL; e = e->next) {
+
+		for (e = tmpsyntax->headers; e != NULL; e = e->next) {
 		    bool not_compiled = (e->ext == NULL);
+
 		    if (not_compiled) {
 			e->ext = (regex_t *)nmalloc(sizeof(regex_t));
 			regcomp(e->ext, fixbounds(e->ext_regex), REG_EXTENDED);
 		    }
 #ifdef DEBUG
-		    fprintf(stderr,"Matching regex \"%s\" against \"%s\"\n",e->ext_regex, magicstring);
-#endif /* DEBUG */
+		    fprintf(stderr, "Comparing header regex \"%s\" to fileage \"%s\"...\n",
+				    e->ext_regex, openfile->fileage->data);
+#endif
+		    /* Set colorstrings if we match the header-line regex. */
+		    if (regexec(e->ext, openfile->fileage->data, 0, NULL, 0) == 0) {
+			openfile->syntax = tmpsyntax;
+			openfile->colorstrings = tmpsyntax->color;
+			break;
+		    }
 
+		    if (not_compiled)
+			nfreeregex(&e->ext);
+		}
+	    }
+	}
+
+#ifdef HAVE_LIBMAGIC
+	/* Check magic if we don't have an answer yet. */
+	if (openfile->colorstrings == NULL) {
+#ifdef DEBUG
+	    fprintf(stderr, "No result from headerline either, trying libmagic...\n");
+#endif
+	    if (stat(openfile->filename, &fileinfo) == 0) {
+		/* Open the magic database and get a diagnosis of the file. */
+		cookie = magic_open(MAGIC_SYMLINK |
+#ifdef DEBUG
+				       MAGIC_DEBUG | MAGIC_CHECK |
+#endif
+				       MAGIC_ERROR);
+		if (cookie == NULL || magic_load(cookie, NULL) < 0)
+		    statusbar(_("magic_load() failed: %s"), strerror(errno));
+		else {
+		    magicstring = magic_file(cookie, openfile->filename);
+		    if (magicstring == NULL) {
+			statusbar(_("magic_file(%s) failed: %s"),
+					openfile->filename, magic_error(cookie));
+		    }
+#ifdef DEBUG
+		    fprintf(stderr, "Returned magic string is: %s\n", magicstring);
+#endif
+		}
+	    }
+
+	    /* Now try and find a syntax that matches the magicstring. */
+	    for (tmpsyntax = syntaxes; tmpsyntax != NULL;
+		tmpsyntax = tmpsyntax->next) {
+
+		for (e = tmpsyntax->magics; e != NULL; e = e->next) {
+		    bool not_compiled = (e->ext == NULL);
+
+		    if (not_compiled) {
+			e->ext = (regex_t *)nmalloc(sizeof(regex_t));
+			regcomp(e->ext, fixbounds(e->ext_regex), REG_EXTENDED);
+		    }
+#ifdef DEBUG
+		    fprintf(stderr, "Matching regex \"%s\" against \"%s\"\n", e->ext_regex, magicstring);
+#endif
+		    /* Set colorstrings if we match the magic-string regex. */
 		    if (magicstring && regexec(e->ext, magicstring, 0, NULL, 0) == 0) {
 			openfile->syntax = tmpsyntax;
 			openfile->colorstrings = tmpsyntax->color;
@@ -253,54 +310,17 @@ void color_update(void)
 		    if (not_compiled)
 			nfreeregex(&e->ext);
 		}
+		if (openfile->syntax != NULL)
+		    break;
 	    }
+	    if (stat(openfile->filename, &fileinfo) == 0)
+		magic_close(cookie);
 	}
 #endif /* HAVE_LIBMAGIC */
-
-	/* If we haven't matched anything yet, try the headers */
-	if (openfile->colorstrings == NULL) {
-#ifdef DEBUG
-	    fprintf(stderr, "No match for file extensions, looking at headers...\n");
-#endif
-	    for (tmpsyntax = syntaxes; tmpsyntax != NULL;
-		tmpsyntax = tmpsyntax->next) {
-
-		for (e = tmpsyntax->headers; e != NULL; e = e->next) {
-		    bool not_compiled = (e->ext == NULL);
-
-		    /* e->ext_regex has already been checked for validity
-		     * elsewhere.  Compile its specified regex if we haven't
-		     * already. */
-		    if (not_compiled) {
-			e->ext = (regex_t *)nmalloc(sizeof(regex_t));
-			regcomp(e->ext, fixbounds(e->ext_regex), REG_EXTENDED);
-		    }
-
-		    /* Set colorstrings if we matched the extension
-		     * regex. */
-#ifdef DEBUG
-		fprintf(stderr, "Comparing header regex \"%s\" to fileage \"%s\"...\n", e->ext_regex, openfile->fileage->data);
-#endif
-		    if (regexec(e->ext, openfile->fileage->data, 0, NULL, 0) == 0) {
-			openfile->syntax = tmpsyntax;
-			openfile->colorstrings = tmpsyntax->color;
-		    }
-
-		    if (openfile->colorstrings != NULL)
-			break;
-
-		    /* Decompile e->ext_regex's specified regex if we aren't
-		     * going to use it. */
-		    if (not_compiled)
-			nfreeregex(&e->ext);
-		}
-	    }
-	}
     }
 
-
-    /* If we didn't get a syntax based on the file extension, and we
-     * have a default syntax, use it. */
+    /* If we didn't find any syntax yet, and we do have a default one,
+     * use it. */
     if (openfile->colorstrings == NULL && defcolor != NULL) {
 	openfile->syntax = defsyntax;
 	openfile->colorstrings = defcolor;
@@ -326,7 +346,7 @@ void color_update(void)
 }
 
 /* Reset the multicolor info cache for records for any lines which need
-   to be recalculated */
+ * to be recalculated. */
 void reset_multis_after(filestruct *fileptr, int mindex)
 {
     filestruct *oof;
@@ -372,11 +392,10 @@ void reset_multis_before(filestruct *fileptr, int mindex)
 	else
 	    break;
     }
-
     edit_refresh_needed = TRUE;
 }
 
-/* Reset one multiline regex info */
+/* Reset one multiline regex info. */
 void reset_multis_for_id(filestruct *fileptr, int num)
 {
     reset_multis_before(fileptr, num);
@@ -384,9 +403,9 @@ void reset_multis_for_id(filestruct *fileptr, int num)
     fileptr->multidata[num] = -1;
 }
 
-/* Reset multi line strings around a filestruct ptr, trying to be smart about stopping
-   force = reset everything regardless, useful when we don't know how much screen state
-           has changed  */
+/* Reset multi-line strings around a filestruct ptr, trying to be smart
+ * about stopping.  Bool force means: reset everything regardless, useful
+ * when we don't know how much screen state has changed. */
 void reset_multis(filestruct *fileptr, bool force)
 {
     int nobegin, noend;
@@ -398,7 +417,7 @@ void reset_multis(filestruct *fileptr, bool force)
 
     for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
 
-	/* If it's not a multi-line regex, amscray */
+	/* If it's not a multi-line regex, amscray. */
 	if (tmpcolor->end == NULL)
 	    continue;
 
@@ -409,10 +428,10 @@ void reset_multis(filestruct *fileptr, bool force)
 	}
 
 	/* Figure out where the first begin and end are to determine if
-	   things changed drastically for the precalculated multi values */
-        nobegin = regexec(tmpcolor->start, fileptr->data, 1, &startmatch, 0);
-        noend = regexec(tmpcolor->end, fileptr->data, 1, &endmatch, 0);
-	if (fileptr->multidata[tmpcolor->id] ==  CWHOLELINE) {
+	 * things changed drastically for the precalculated multi values. */
+	nobegin = regexec(tmpcolor->start, fileptr->data, 1, &startmatch, 0);
+	noend = regexec(tmpcolor->end, fileptr->data, 1, &endmatch, 0);
+	if (fileptr->multidata[tmpcolor->id] == CWHOLELINE) {
 	    if (nobegin && noend)
 		continue;
 	} else if (fileptr->multidata[tmpcolor->id] == CNONE) {
@@ -424,8 +443,9 @@ void reset_multis(filestruct *fileptr, bool force)
 	    continue;
 	}
 
-	/* If we got here assume the worst */
+	/* If we got here, assume the worst. */
 	reset_multis_for_id(fileptr, tmpcolor->id);
     }
 }
-#endif /* ENABLE_COLOR */
+
+#endif /* !DISABLE_COLOR */
